import os
from unittest import mock

import pytest


pytest.importorskip("IPython")


@mock.patch.dict(os.environ, {}, clear=True)
def test_ipython_existing_variable_no_override(tmp_path):
    from IPython.terminal.embed import InteractiveShellEmbed

    dotenv_file = tmp_path / ".env"
    dotenv_file.write_text("a=b\n")
    os.chdir(tmp_path)
    os.environ["a"] = "c"

    ipshell = InteractiveShellEmbed()
    ipshell.run_line_magic("load_ext", "dotenv")
    ipshell.run_line_magic("dotenv", "")

    assert os.environ == {"a": "c"}


@mock.patch.dict(os.environ, {}, clear=True)
def test_ipython_existing_variable_override(tmp_path):
    from IPython.terminal.embed import InteractiveShellEmbed

    dotenv_file = tmp_path / ".env"
    dotenv_file.write_text("a=b\n")
    os.chdir(tmp_path)
    os.environ["a"] = "c"

    ipshell = InteractiveShellEmbed()
    ipshell.run_line_magic("load_ext", "dotenv")
    ipshell.run_line_magic("dotenv", "-o")

    assert os.environ == {"a": "b"}


@mock.patch.dict(os.environ, {}, clear=True)
def test_ipython_new_variable(tmp_path):
    from IPython.terminal.embed import InteractiveShellEmbed

    dotenv_file = tmp_path / ".env"
    dotenv_file.write_text("a=b\n")
    os.chdir(tmp_path)

    ipshell = InteractiveShellEmbed()
    ipshell.run_line_magic("load_ext", "dotenv")
    ipshell.run_line_magic("dotenv", "")

    assert os.environ == {"a": "b"}



import io

import pytest

from dotenv.parser import Binding, Original, parse_stream


@pytest.mark.parametrize("test_input,expected", [
    (u"", []),
    (u"a=b", [Binding(key=u"a", value=u"b", original=Original(string=u"a=b", line=1), error=False)]),
    (u"'a'=b", [Binding(key=u"a", value=u"b", original=Original(string=u"'a'=b", line=1), error=False)]),
    (u"[=b", [Binding(key=u"[", value=u"b", original=Original(string=u"[=b", line=1), error=False)]),
    (u" a = b ", [Binding(key=u"a", value=u"b", original=Original(string=u" a = b ", line=1), error=False)]),
    (u"export a=b", [Binding(key=u"a", value=u"b", original=Original(string=u"export a=b", line=1), error=False)]),
    (
        u" export 'a'=b",
        [Binding(key=u"a", value=u"b", original=Original(string=u" export 'a'=b", line=1), error=False)],
    ),
    (u"# a=b", [Binding(key=None, value=None, original=Original(string=u"# a=b", line=1), error=False)]),
    (u"a=b#c", [Binding(key=u"a", value=u"b#c", original=Original(string=u"a=b#c", line=1), error=False)]),
    (
        u'a=b #c',
        [Binding(key=u"a", value=u"b", original=Original(string=u"a=b #c", line=1), error=False)],
    ),
    (
        u'a=b\t#c',
        [Binding(key=u"a", value=u"b", original=Original(string=u"a=b\t#c", line=1), error=False)],
    ),
    (
        u"a=b c",
        [Binding(key=u"a", value=u"b c", original=Original(string=u"a=b c", line=1), error=False)],
    ),
    (
        u"a=b\tc",
        [Binding(key=u"a", value=u"b\tc", original=Original(string=u"a=b\tc", line=1), error=False)],
    ),
    (
        u"a=b  c",
        [Binding(key=u"a", value=u"b  c", original=Original(string=u"a=b  c", line=1), error=False)],
    ),
    (
        u"a=b\u00a0 c",
        [Binding(key=u"a", value=u"b\u00a0 c", original=Original(string=u"a=b\u00a0 c", line=1), error=False)],
    ),
    (
        u"a=b c ",
        [Binding(key=u"a", value=u"b c", original=Original(string=u"a=b c ", line=1), error=False)],
    ),
    (
        u"a='b c '",
        [Binding(key=u"a", value=u"b c ", original=Original(string=u"a='b c '", line=1), error=False)],
    ),
    (
        u'a="b c "',
        [Binding(key=u"a", value=u"b c ", original=Original(string=u'a="b c "', line=1), error=False)],
    ),
    (
        u"export export_a=1",
        [
            Binding(key=u"export_a", value=u"1", original=Original(string=u"export export_a=1", line=1), error=False)
        ],
    ),
    (
        u"export port=8000",
        [Binding(key=u"port", value=u"8000", original=Original(string=u"export port=8000", line=1), error=False)],
    ),
    (u'a="b\nc"', [Binding(key=u"a", value=u"b\nc", original=Original(string=u'a="b\nc"', line=1), error=False)]),
    (u"a='b\nc'", [Binding(key=u"a", value=u"b\nc", original=Original(string=u"a='b\nc'", line=1), error=False)]),
    (u'a="b\nc"', [Binding(key=u"a", value=u"b\nc", original=Original(string=u'a="b\nc"', line=1), error=False)]),
    (u'a="b\\nc"', [Binding(key=u"a", value=u'b\nc', original=Original(string=u'a="b\\nc"', line=1), error=False)]),
    (u"a='b\\nc'", [Binding(key=u"a", value=u'b\\nc', original=Original(string=u"a='b\\nc'", line=1), error=False)]),
    (u'a="b\\"c"', [Binding(key=u"a", value=u'b"c', original=Original(string=u'a="b\\"c"', line=1), error=False)]),
    (u"a='b\\'c'", [Binding(key=u"a", value=u"b'c", original=Original(string=u"a='b\\'c'", line=1), error=False)]),
    (u"a=à", [Binding(key=u"a", value=u"à", original=Original(string=u"a=à", line=1), error=False)]),
    (u'a="à"', [Binding(key=u"a", value=u"à", original=Original(string=u'a="à"', line=1), error=False)]),
    (
        u'no_value_var',
        [Binding(key=u'no_value_var', value=None, original=Original(string=u"no_value_var", line=1), error=False)],
    ),
    (u'a: b', [Binding(key=None, value=None, original=Original(string=u"a: b", line=1), error=True)]),
    (
        u"a=b\nc=d",
        [
            Binding(key=u"a", value=u"b", original=Original(string=u"a=b\n", line=1), error=False),
            Binding(key=u"c", value=u"d", original=Original(string=u"c=d", line=2), error=False),
        ],
    ),
    (
        u"a=b\rc=d",
        [
            Binding(key=u"a", value=u"b", original=Original(string=u"a=b\r", line=1), error=False),
            Binding(key=u"c", value=u"d", original=Original(string=u"c=d", line=2), error=False),
        ],
    ),
    (
        u"a=b\r\nc=d",
        [
            Binding(key=u"a", value=u"b", original=Original(string=u"a=b\r\n", line=1), error=False),
            Binding(key=u"c", value=u"d", original=Original(string=u"c=d", line=2), error=False),
        ],
    ),
    (
        u'a=\nb=c',
        [
            Binding(key=u"a", value=u'', original=Original(string=u'a=\n', line=1), error=False),
            Binding(key=u"b", value=u'c', original=Original(string=u"b=c", line=2), error=False),
        ]
    ),
    (
        u"\n\n",
        [
            Binding(key=None, value=None, original=Original(string=u"\n\n", line=1), error=False),
        ]
    ),
    (
        u"a=b\n\n",
        [
            Binding(key=u"a", value=u"b", original=Original(string=u"a=b\n", line=1), error=False),
            Binding(key=None, value=None, original=Original(string=u"\n", line=2), error=False),
        ]
    ),
    (
        u'a=b\n\nc=d',
        [
            Binding(key=u"a", value=u"b", original=Original(string=u"a=b\n", line=1), error=False),
            Binding(key=u"c", value=u"d", original=Original(string=u"\nc=d", line=2), error=False),
        ]
    ),
    (
        u'a="\nb=c',
        [
            Binding(key=None, value=None, original=Original(string=u'a="\n', line=1), error=True),
            Binding(key=u"b", value=u"c", original=Original(string=u"b=c", line=2), error=False),
        ]
    ),
    (
        u'# comment\na="b\nc"\nd=e\n',
        [
            Binding(key=None, value=None, original=Original(string=u"# comment\n", line=1), error=False),
            Binding(key=u"a", value=u"b\nc", original=Original(string=u'a="b\nc"\n', line=2), error=False),
            Binding(key=u"d", value=u"e", original=Original(string=u"d=e\n", line=4), error=False),
        ],
    ),
    (
        u'a=b\n# comment 1',
        [
            Binding(key="a", value="b", original=Original(string=u"a=b\n", line=1), error=False),
            Binding(key=None, value=None, original=Original(string=u"# comment 1", line=2), error=False),
        ],
    ),
    (
        u'# comment 1\n# comment 2',
        [
            Binding(key=None, value=None, original=Original(string=u"# comment 1\n", line=1), error=False),
            Binding(key=None, value=None, original=Original(string=u"# comment 2", line=2), error=False),
        ],
    ),
    (
        u'uglyKey[%$=\"S3cr3t_P4ssw#rD\" #\na=b',
        [
            Binding(key=u'uglyKey[%$',
                    value=u'S3cr3t_P4ssw#rD',
                    original=Original(string=u"uglyKey[%$=\"S3cr3t_P4ssw#rD\" #\n", line=1), error=False),
            Binding(key=u"a", value=u"b", original=Original(string=u'a=b', line=2), error=False),
        ],
    ),
])
def test_parse_stream(test_input, expected):
    result = parse_stream(io.StringIO(test_input))

    assert list(result) == expected



from setuptools import setup


def read_files(files):
    data = []
    for file in files:
        with open(file, encoding='utf-8') as f:
            data.append(f.read())
    return "\n".join(data)


long_description = read_files(['README.md', 'CHANGELOG.md'])

meta = {}
with open('./src/dotenv/version.py', encoding='utf-8') as f:
    exec(f.read(), meta)

setup(
    name="python-dotenv",
    description="Read key-value pairs from a .env file and set them as environment variables",
    long_description=long_description,
    long_description_content_type='text/markdown',
    version=meta['__version__'],
    author="Saurabh Kumar",
    author_email="me+github@saurabh-kumar.com",
    url="https://github.com/theskumar/python-dotenv",
    keywords=['environment variables', 'deployments', 'settings', 'env', 'dotenv',
              'configurations', 'python'],
    packages=['dotenv'],
    package_dir={'': 'src'},
    package_data={
        'dotenv': ['py.typed'],
    },
    python_requires=">=3.8",
    extras_require={
        'cli': ['click>=5.0', ],
    },
    entry_points={
        "console_scripts": [
            "dotenv=dotenv.__main__:cli",
        ],
    },
    license='BSD-3-Clause',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities',
        'Environment :: Web Environment',
    ]
)
